from pydantic import BaseModel
from typing import List, Optional
from sqlalchemy import Column, String, Integer, Float, JSON, DateTime
from datetime import datetime, timezone
from .base import ExchangeInfoSymbol

# ✅ Pydantic Model
class BinanceFutureUSDTExchangeInfoSymbol(BaseModel):
    symbol: str
    pair: str
    contractType: str
    deliveryDate: Optional[datetime]
    onboardDate: Optional[datetime]
    status: str
    baseAsset: str
    quoteAsset: str
    marginAsset: str
    pricePrecision: int
    quantityPrecision: int
    baseAssetPrecision: int
    quotePrecision: int
    underlyingType: str
    underlyingSubType: List[str]
    settlePlan: int
    triggerProtect: str
    filters: List[dict]
    OrderType: List[str]
    timeInForce: List[str]
    liquidationFee: str
    marketTakeBound: str


# ✅ ORM Table
class BinanceFutureUSDTExchangeInfoSymbolTable(ExchangeInfoSymbol):
    __tablename__ = "binance_future_usdt_exchange_info_symbols"

    id = Column(Integer, primary_key=True, autoincrement=True)
    symbol = Column(String, index=True, unique=True)
    pair = Column(String, index=True)
    contractType = Column(String)
    deliveryDate = Column(DateTime(timezone=True)) 
    onboardDate = Column(DateTime(timezone=True)) 
    status = Column(String)
    baseAsset = Column(String)
    quoteAsset = Column(String)
    marginAsset = Column(String)
    pricePrecision = Column(Integer)
    quantityPrecision = Column(Integer)
    baseAssetPrecision = Column(Integer)
    quotePrecision = Column(Integer)
    underlyingType = Column(String)
    underlyingSubType = Column(JSON)
    settlePlan = Column(Integer)
    triggerProtect = Column(String)
    filters = Column(JSON)
    OrderType = Column(JSON)
    timeInForce = Column(JSON)
    liquidationFee = Column(String)
    marketTakeBound = Column(String)

class BinanceFutureUSDTExchangeInfoSymbolMapper:
    @staticmethod
    def from_raw(raw: dict) -> BinanceFutureUSDTExchangeInfoSymbol:
        return BinanceFutureUSDTExchangeInfoSymbol(
            symbol=raw["symbol"],
            pair=raw["pair"],
            contractType=raw["contractType"],
            deliveryDate=datetime.fromtimestamp(raw["deliveryDate"]/1000, tz=timezone.utc)
                          if raw.get("deliveryDate") else None,
            onboardDate=datetime.fromtimestamp(raw["onboardDate"]/1000, tz=timezone.utc)
                         if raw.get("onboardDate") else None,
            status=raw["status"],
            baseAsset=raw["baseAsset"],
            quoteAsset=raw["quoteAsset"],
            marginAsset=raw["marginAsset"],
            pricePrecision=raw["pricePrecision"],
            quantityPrecision=raw["quantityPrecision"],
            baseAssetPrecision=raw["baseAssetPrecision"],
            quotePrecision=raw["quotePrecision"],
            underlyingType=raw["underlyingType"],
            underlyingSubType=raw.get("underlyingSubType", []),
            settlePlan=raw.get("settlePlan", 0),
            triggerProtect=raw.get("triggerProtect", "0"),
            filters=raw.get("filters", []),
            OrderType=raw.get("OrderType", []),
            timeInForce=raw.get("timeInForce", []),
            liquidationFee=raw.get("liquidationFee", "0"),
            marketTakeBound=raw.get("marketTakeBound", "0"),
        )

    @staticmethod
    def to_table(event: BinanceFutureUSDTExchangeInfoSymbol) -> BinanceFutureUSDTExchangeInfoSymbolTable:
        return BinanceFutureUSDTExchangeInfoSymbolTable(
            symbol=event.symbol,
            pair=event.pair,
            contractType=event.contractType,
            deliveryDate=event.deliveryDate,
            onboardDate=event.onboardDate,
            status=event.status,
            baseAsset=event.baseAsset,
            quoteAsset=event.quoteAsset,
            marginAsset=event.marginAsset,
            pricePrecision=event.pricePrecision,
            quantityPrecision=event.quantityPrecision,
            baseAssetPrecision=event.baseAssetPrecision,
            quotePrecision=event.quotePrecision,
            underlyingType=event.underlyingType,
            underlyingSubType=event.underlyingSubType,
            settlePlan=event.settlePlan,
            triggerProtect=event.triggerProtect,
            filters=event.filters,
            OrderType=event.OrderType,
            timeInForce=event.timeInForce,
            liquidationFee=event.liquidationFee,
            marketTakeBound=event.marketTakeBound,
        )



from pydantic import BaseModel
from datetime import datetime, timezone
from typing import List, Optional
from sqlalchemy import Column, String, Boolean, Integer, JSON
from .base import ExchangeInfoSymbol

# ✅ Pydantic Model
class BinanceSpotExchangeInfoSymbol(BaseModel):
    symbol: str
    status: str
    baseAsset: str
    baseAssetPrecision: int
    quoteAsset: str
    quotePrecision: int
    quoteAssetPrecision: int
    baseCommissionPrecision: int
    quoteCommissionPrecision: int
    orderTypes: List[str]
    icebergAllowed: bool
    ocoAllowed: bool
    otoAllowed: bool
    quoteOrderQtyMarketAllowed: bool
    allowTrailingStop: bool
    cancelReplaceAllowed: bool
    amendAllowed: bool
    pegInstructionsAllowed: bool
    isSpotTradingAllowed: bool
    isMarginTradingAllowed: bool
    filters: Optional[List[dict]] = []
    permissions: Optional[List[str]] = []
    permissionSets: Optional[List[List[str]]] = []
    defaultSelfTradePreventionMode: Optional[str]
    allowedSelfTradePreventionModes: Optional[List[str]] = []

# ✅ ORM Table
class BinanceSpotExchangeInfoSymbolTable(ExchangeInfoSymbol):
    __tablename__ = "binance_spot_exchange_info_symbols"

    id = Column(Integer, primary_key=True, autoincrement=True)
    symbol = Column(String, index=True, unique=True)
    status = Column(String)
    baseAsset = Column(String)
    baseAssetPrecision = Column(Integer)
    quoteAsset = Column(String)
    quotePrecision = Column(Integer)
    quoteAssetPrecision = Column(Integer)
    baseCommissionPrecision = Column(Integer)
    quoteCommissionPrecision = Column(Integer)
    orderTypes = Column(JSON)
    icebergAllowed = Column(Boolean)
    ocoAllowed = Column(Boolean)
    otoAllowed = Column(Boolean)
    quoteOrderQtyMarketAllowed = Column(Boolean)
    allowTrailingStop = Column(Boolean)
    cancelReplaceAllowed = Column(Boolean)
    amendAllowed = Column(Boolean)
    pegInstructionsAllowed = Column(Boolean)
    isSpotTradingAllowed = Column(Boolean)
    isMarginTradingAllowed = Column(Boolean)
    filters = Column(JSON)
    permissions = Column(JSON)
    permissionSets = Column(JSON)
    defaultSelfTradePreventionMode = Column(String)
    allowedSelfTradePreventionModes = Column(JSON)


# ✅ Mapper
class BinanceSpotExchangeInfoSymbolMapper:
    @staticmethod
    def from_raw(raw: dict) -> BinanceSpotExchangeInfoSymbol:
        return BinanceSpotExchangeInfoSymbol(
            symbol=raw["symbol"],
            status=raw["status"],
            baseAsset=raw["baseAsset"],
            baseAssetPrecision=raw["baseAssetPrecision"],
            quoteAsset=raw["quoteAsset"],
            quotePrecision=raw.get("quotePrecision", 0),
            quoteAssetPrecision=raw["quoteAssetPrecision"],
            baseCommissionPrecision=raw["baseCommissionPrecision"],
            quoteCommissionPrecision=raw["quoteCommissionPrecision"],
            orderTypes=raw.get("orderTypes", []),
            icebergAllowed=raw.get("icebergAllowed", False),
            ocoAllowed=raw.get("ocoAllowed", False),
            otoAllowed=raw.get("otoAllowed", False),
            quoteOrderQtyMarketAllowed=raw.get("quoteOrderQtyMarketAllowed", False),
            allowTrailingStop=raw.get("allowTrailingStop", False),
            cancelReplaceAllowed=raw.get("cancelReplaceAllowed", False),
            amendAllowed=raw.get("amendAllowed", False),
            pegInstructionsAllowed=raw.get("pegInstructionsAllowed", False),
            isSpotTradingAllowed=raw.get("isSpotTradingAllowed", False),
            isMarginTradingAllowed=raw.get("isMarginTradingAllowed", False),
            filters=raw.get("filters", []),
            permissions=raw.get("permissions", []),
            permissionSets=raw.get("permissionSets", []),
            defaultSelfTradePreventionMode=raw.get("defaultSelfTradePreventionMode"),
            allowedSelfTradePreventionModes=raw.get("allowedSelfTradePreventionModes", []),
        )

    @staticmethod
    def to_table(event: BinanceSpotExchangeInfoSymbol) -> BinanceSpotExchangeInfoSymbolTable:
        return BinanceSpotExchangeInfoSymbolTable(
            symbol=event.symbol,
            status=event.status,
            baseAsset=event.baseAsset,
            baseAssetPrecision=event.baseAssetPrecision,
            quoteAsset=event.quoteAsset,
            quotePrecision=event.quotePrecision,
            quoteAssetPrecision=event.quoteAssetPrecision,
            baseCommissionPrecision=event.baseCommissionPrecision,
            quoteCommissionPrecision=event.quoteCommissionPrecision,
            orderTypes=event.orderTypes,
            icebergAllowed=event.icebergAllowed,
            ocoAllowed=event.ocoAllowed,
            otoAllowed=event.otoAllowed,
            quoteOrderQtyMarketAllowed=event.quoteOrderQtyMarketAllowed,
            allowTrailingStop=event.allowTrailingStop,
            cancelReplaceAllowed=event.cancelReplaceAllowed,
            amendAllowed=event.amendAllowed,
            pegInstructionsAllowed=event.pegInstructionsAllowed,
            isSpotTradingAllowed=event.isSpotTradingAllowed,
            isMarginTradingAllowed=event.isMarginTradingAllowed,
            filters=event.filters,
            permissions=event.permissions,
            permissionSets=event.permissionSets,
            defaultSelfTradePreventionMode=event.defaultSelfTradePreventionMode,
            allowedSelfTradePreventionModes=event.allowedSelfTradePreventionModes,
        )



from pydantic import BaseModel
from datetime import datetime, timezone
from sqlalchemy import Column, String, Float, Boolean, Integer, DateTime
from .base import StreamKline

class BinanceStreamFuturesCOINMKline(BaseModel):
    source: str = "binance"
    type: str = "kline"
    market: str = "futures_coinm"
    symbol: str
    interval: str
    event_time: datetime
    open_time: datetime
    close_time: datetime
    is_closed: bool
    open: float
    high: float
    low: float
    close: float
    volume: float
    quote_volume: float
    trades: int
    taker_buy_base: float
    taker_buy_quote: float

# ✅ ORM Table
class BinanceStreamFuturesCOINMKlineTable(StreamKline):
    __tablename__ = "binance_futures_coinm_kline"

    id = Column(Integer, primary_key=True, autoincrement=True)
    symbol = Column(String, index=True)
    interval = Column(String)
    event_time = Column(DateTime, index=True)
    open_time = Column(DateTime)
    close_time = Column(DateTime)
    is_closed = Column(Boolean)
    open = Column(Float)
    high = Column(Float)
    low = Column(Float)
    close = Column(Float)
    volume = Column(Float)
    quote_volume = Column(Float)
    trades = Column(Integer)
    taker_buy_base = Column(Float)
    taker_buy_quote = Column(Float)

# ✅ util แปลง timestamp
def ms_to_dt(ms: int) -> datetime:
    return datetime.fromtimestamp(ms/1000, tz=timezone.utc)

# ✅ Mapper
class BinanceStreamFuturesCOINMKlineMapper:
    @staticmethod
    def from_raw(raw: dict) -> BinanceStreamFuturesCOINMKline:
        d = raw.get("data") or raw
        k = d.get("k") or {}

        return BinanceStreamFuturesCOINMKline(
            symbol=d["s"],
            interval=k["i"],
            event_time=ms_to_dt(d["E"]),
            open_time=ms_to_dt(k["t"]),
            close_time=ms_to_dt(k["T"]),
            is_closed=k["x"],
            open=float(k["o"]),
            high=float(k["h"]),
            low=float(k["l"]),
            close=float(k["c"]),
            volume=float(k["v"]),
            quote_volume=float(k["q"]),
            trades=int(k["n"]),
            taker_buy_base=float(k["V"]),
            taker_buy_quote=float(k["Q"])
        )

    @staticmethod
    def to_table(event: BinanceStreamFuturesCOINMKline) -> BinanceStreamFuturesCOINMKlineTable:
        return BinanceStreamFuturesCOINMKlineTable(
            symbol=event.symbol,
            interval=event.interval,
            event_time=event.event_time,
            open_time=event.open_time,
            close_time=event.close_time,
            is_closed=event.is_closed,
            open=event.open,
            high=event.high,
            low=event.low,
            close=event.close,
            volume=event.volume,
            quote_volume=event.quote_volume,
            trades=event.trades,
            taker_buy_base=event.taker_buy_base,
            taker_buy_quote=event.taker_buy_quote,
        )
